//////////////////////////////////////////////////////////////////
// Symbolic
//
// Library for the construction of nested symbolic propositions
// The Flying Pig!
// Started 5/8/2003
// http://www.flypig.co.uk
// 
// 
// The MIT License
// See symbolic.h, COPYING file or website for licence
// 
//////////////////////////////////////////////////////////////////

#if !defined _H_SYMBOLIC_PRIVATE
#define _H_SYMBOLIC_PRIVATE

//////////////////////////////////////////////////////////////////
// Includes

#if HAVE_CONFIG_H
#include <config.h>
#endif

//////////////////////////////////////////////////////////////////
// Defines

#define WORDALIGN(b)    (((b) + 3) & ~3)

#define MAXVARSIZE      (64)

#if defined _DEBUG
#define DPRINTF         printf
#else
#define DPRINTF         noprintf
#endif

#if defined _DEBUG
#define REPORT                Report
#define REPORTVAR             ReportVar
void    Report                (char * szMessage);
void    ReportVar             (char * szFormat, int nVariable);
#else
#define REPORT                //
#define REPORTVAR             //
#endif

#if defined _RISC_OS
#define _snprintf             snprintf
#endif

#if defined _MEM_PROFILE
#define PropMalloc            PropMemMalloc
#define PropCalloc            PropMemCalloc
#define PropFree              PropMemFree
#else
#define PropMalloc            malloc
#define PropCalloc            calloc
#define PropFree              free
#endif // if defined _MEM_PROFILE

#if defined (WIN32) && !defined (NAN)
static const unsigned long nan[2] = {0xffffffff, 0x7fffffff};
#define NAN (*(const double *) nan)
#endif

#if defined (WIN32)
#define snprintf _snprintf
#define snscanf _snscanf
#endif

//////////////////////////////////////////////////////////////////
// Structures

typedef enum _OPTYPE {
  OPTYPE_INVALID = -1,

  OPTYPE_MEMORY,
  OPTYPE_INTEGER,
  OPTYPE_TRUTHVALUE,
  OPTYPE_VARIABLE,
  OPTYPE_UNARY,
  OPTYPE_BINARY,
  OPTYPE_TERNARY,

  OPTYPE_NUM
} OPTYPE;

typedef struct _OpUnary {
  OPUNARY       eOpType;
  Operation     *psVar1;
} OpUnary;

typedef struct _OpBinary {
  OPBINARY      eOpType;
  Operation     *psVar1;
  Operation     *psVar2;
} OpBinary;

typedef struct _OpTernary {
  OPTERNARY     eOpType;
  Operation     *psVar1;
  Operation     *psVar2;
  Operation     *psVar3;
} OpTernary;

typedef struct _OpVariable {
  char          szVar[MAXVARSIZE];
  Variable      *psValue;
} OpVariable;

struct _Operation {
  OPTYPE        eOpType;
  union
  {
    bool        boTruth;
    int         nInteger;
    OpVariable  *psVar;
    OpUnary     *psUnary;
    OpBinary    *psBinary;
    OpTernary   *psTernary;
  } Vars;
};

//////////////////////////////////////////////////////////////////
// Function prototypes

void          DecrementVarRef         (Variable * psVar);
void          IncrementVarRef         (Variable * psVar);

#endif // if !defined _H_SYMBOLIC_PRIVATE
